
%macro makeCharts( data=_last_, response=, p=, event=1, plot=, groups=20, grid=on, xaxis=selectedPct, 
	out=_chart, path=, filename=output );

	%local area ks n0 n1 name1 nlvls nobs ylabel _opts_ _last_ _title _title2 _title3 _xvar _yvar;

	%if &data = _last_ %then %let data=&syslast;
	%let _opts = %sysfunc( getoption( notes ) ) _last_ = %sysfunc( getoption( _last_ ) );	
	%str( options nonotes; );

	%* DATA= data set option must be specified and must exist;
	%if &data= or %sysfunc( exist( &data ) ) ne 1 %then %do;
		%put ERROR: DATA= data set not specified or not found.;
		%goto exit;
  	%end;

	%* RESPONSE= check;
	%if &response= %then %do;
		%put ERROR: The RESPONSE= parameter is required.;
		%goto exit;
 		%end;

	%* P= check;
	%if &p= %then %do;
		%put ERROR: The P= parameter is required.;
		%goto exit;
 		%end;

	%* EVENT= check;
	%if &event= %then %do;
		%put ERROR: The EVENT= parameter is required.;
		%goto exit;
 		%end;

	%* GROUPS= check;
	%if &groups ne 10 and &groups ne 20 %then %do;
		%put ERROR: GROUPS= must be 10 or 20;
		%goto exit;
		%end;

	%* GRID= check;
	%if %upcase( &grid ) ne ON and %upcase( &grid ) ne OFF %then %do;
		%put ERROR: GRID= must be ON or OFF;
		%goto exit;
		%end;

	%* XAXIS check;
	%if %upcase( %substr( &xaxis, 1, 1 ) ) = G %then %let _xvar=_grp;
		%else %if %upcase( %substr( &xaxis, 1, 1 ) ) = S %then %let _xvar=selectedPct;
		%else %if %upcase( %substr( &xaxis, 1, 1 ) ) = P %then %let _xvar=percentile;
		%else %do;
			%put ERROR: XAXIS= must be GROUPNUM or PERCENTILE or SELECTEDPCT;
			%goto exit;
			%end;

	ods exclude all;
	ods output nlevels=_nlvls;

	proc freq data=&data nlevels;
		where missing( &response ) ne 1;
		table &response / out=freqs;
	run;

	data _null_;
		set freqs;
		if &response = 0 then call symput( 'n0', count );
			else if &response = 1 then call symput( 'n1', count );
	run;

	%let nobs = %eval( &n0 + &n1 );
	%let n1 = %sysfunc( putn( &n1, comma12. ) );
	  %* puts in the comma for big numbers, gets rid of blank spaces around it;
	%let n0 = %sysfunc( putn( &n0, comma12. ) );
	%let nobs = %sysfunc( putn( &nobs, comma12. ) );

	ods select all;

	data _null_;
		set _nlvls;
		call symput( "nlvls", nlevels );
	run;

	%if &nlvls ne 2 %then %do;
		%put ERROR: Response variable, &response, must have exactly two levels.;
		%goto exit;
		%end;

	%if %upcase( %substr( &plot, 1, 2 ) ) ne KS %then %do;

		proc rank data=&data out=_ranks groups=&groups;
			var &p; 
			ranks _rp;
		run;

		%end;

	data _ranks;
		set %if %upcase( %substr( &plot, 1, 2 ) ) = KS %then &data;
			%else _ranks;;
		%if %upcase( %substr( &plot, 1, 2 ) ) = KS %then %do;
				_grp = ceil( &groups*&p  );
				if _grp = 0 then _grp = 1; %* Mapping a probability of 0 to 1, not 0;
				_rp = &groups - _grp;
				label _grp = 'Probability Score Group';
				%end;
			%else %do;
				_grp = &groups - _rp;   
				%end;
		if &response=&event then _y=1;
			else _y=0;               
	run;

	proc summary data=_ranks;
		class _grp;
		var _y;
		output out=_EvntProp mean=EvntProp sum=EvntCnt;
	run;

	data _missgrps;                     * Add any missing groups and set;
		do _grp=.,1 to &groups;            * statistics to zero in missing groups;
			freq_=0; 
			evntProp=0;
			evntCnt=0; 
			output;
			end;
	run;

	data _evntProp;
		merge _missgrps _EvntProp;
		by _grp;
	run;

	data &out;
		set _evntProp nobs=_n;
		retain nobs TotEvntpct TotEvntCnt NumGroups ks area cumPctCaptured_o;
		if _n_=1 then do;
				nobs = _freq_; 
				totEvntPct = EvntProp*100; 
				totEvntCnt = EvntCnt;
				numGroups = _n-1;
				ks = 0;
				area = 0;
				cumPctCaptured_o = 0;
				cumPctCaptured = 0;
				delete;
				end;
			else do;
				selectedPct = _grp*100 / NumGroups;
				percentile = 100 - ( _grp-1 )*100 / NumGroups; 
				cumEvntCnt + EvntCnt;
				cumGrpCnt + _freq_;

		    %* Percent Response;
				pctResp = 100*evntProp;                  
    		basePctResp = totEvntPct;                

				%* Cumulative Percent Response;
				if CumGrpCnt = 0 then cumPctResp=0; 
					else cumPctResp = 100*(CumEvntCnt/CumGrpCnt); 
				baseCumPctResp = totEvntPct;             

				%* Percent Captured;
				pctCaptured = 100*( evntCnt / totEvntCnt );
				basePctCaptured = 100*_freq_ / nobs;

				%* Cumulative Percent Captured;
				cumPctCaptured = 100*( cumEvntCnt / totEvntCnt );
				baseCumPctCaptured = 100*cumGrpCnt / nobs;

				%* Lift;
				lift = pctResp / totEvntPct;                   
				baseLift=1;
    
		    %* Cumulative Lift;
				cumLift = cumPctResp / totEvntPct;             
				baseCumLift = 1;

		    %* Gain;
				gain = 100*( lift-1 );                         
				baseGain = 0;

				%* Cumulative Gain;
				cumGain = 100*( cumLift-1 );                   
				baseCumGain = 0;

				%* Kolmogorov-Smirnov;
				ksEvent = cumPctCaptured; %*100*(CumEvntCnt/TotEvntCnt);
				ksNonEvent = 100*( ( cumGrpCnt - cumEvntCnt ) / ( nobs - totEvntCnt ) );
				ks = max( ks, abs( ksEvent - ksNonEvent ) );

				area + 0.5*( cumPctCaptured_o + cumPctCaptured )/&groups;
				cumPctCaptured_o = cumPctCaptured;

		    drop _type_;
  	 		end;
			label 
				selectedpct = "Population Percent"
				cumLift = "Cumulative Lift"
				pctCaptured="Percent Captured"
				_grp="Group";
	run;

%*	proc freq data=&out noprint;
%*	run;

	data _null_;
		set &out end=eof;
		if eof then do;
			call symput( 'area', round( area/100, 0.0001 ) );
			call symput( 'ks', round( ks, 0.01 ) );
			end;
	run;

	%let area = &area;

	data _add00;
		_grp = 0;
		cumPctCaptured = 0;
		ksEvent = 0;
		ksNonEvent = 0;
		baseCumPctCaptured = 0;
		baseCumLift = 1;
		selectedPct = 0;
	run;

	data &out;
		set _add00 &out;
	run;

	%let ks = &ks;
	%let name1 = Baseline;
	%let ylabel = % Captured;
	%if %upcase( %substr( &plot, 1, 2 ) ) = KS %then %let name1 = Non-Target;;
	%if %upcase( %substr( &plot, 1, 2 ) ) = LI %then %let ylabel = Lift;
	%let _title2 = n=&nobs  (n0=&n0, n1=&n1);

	proc template;
		define statgraph gainLiftPlot;
		mvar _title _title2 _title3 _yvar _xvar _baseline;
			begingraph;
				entrytitle textattrs=( size=14pt ) _title;
				entrytitle textattrs=( weight=normal size=10pt ) _title2;
				%if ( %upcase( %substr( &plot, 1, 2 ) ) = KS or %upcase( %substr( &plot, 1, 2 ) ) = GA ) %then entrytitle textattrs=( weight=normal size=10pt ) _title3;;
					layout overlay / xaxisopts=( griddisplay=&grid )
						yaxisopts=( display=( line ticks tickvalues label ) griddisplay=&grid label="&ylabel" );
					seriesplot y=_baseline x=_xvar / name="Baseline" legendlabel="&name1" lineattrs=GraphData2;
					seriesplot y=_yvar x=_xvar / 
					%if %upcase( %substr( &plot, 1, 2 ) )=KS %then name="Baseline2" legendlabel="Target"; lineattrs=GraphData1; 
					discretelegend "Baseline" %if %upcase( %substr( &plot, 1, 2 ) ) = KS %then "Baseline2";;
					endlayout;
				endgraph;
			end;
		run;

	%if %upcase( %substr( &plot, 1, 2 ) ) = LI %then %do;
			%let _title = "Lift Chart for variable %upcase( &response )";
			%let _yvar = cumLift;
			%let _baseline = baseCumLift;
			%end;
		%else %if %upcase( %substr( &plot, 1, 2 ) ) = GA %then %do;
			%let _title = "Gain Chart for variable %upcase( &response )";
			%let _title3 = "Area under Curve = &area";
			%let _yvar = cumPctCaptured;
			%let _baseline = baseCumPctCaptured;
			%end;
		%else %if %upcase( %substr( &plot, 1, 2 ) ) = KS %then %do;
			%let _title = "KS Chart for variable %upcase( &response )";
			%let _title3 = "KS Statistic=&ks"; 
			%let _yvar = ksEvent;
			%let _baseline = ksNonEvent;
			%end;
		%else %do;
			%put ERROR: PLOT must be GAIN, KS, or LIFT.;
			%goto exit;
			%end;

	ods listing gpath="&path";
		*path for the PNG files from SGRENDER below;

	ods graphics / reset=all imagename="&filename";

	proc sgrender data=&out template=gainLiftPlot;
	run;

	ods graphics off;

	%exit:
	options &_opts;

%mend makeCharts;

