
%macro makeROC( response, outroc=, out=, p=, grid=n, round=1e-8, path=, filename=output );

	%local area nomatch nobs n0 n1 title;

	%let opts = %sysfunc( getoption( notes ) );
	%*%str(options nonotes;%*);

	%let nomatch=0;

	%* Verify P= is specified;
	%if %quote( &p )= %then %do;
  	%put ERROR: The P= option is required.;
  	%goto exit;
		%end;

	%* Verify OUTROC= is specified and the data set exists;
	%if &outroc ne %then %do;
			%if %sysfunc( exist( &outroc ) ) ne 1 %then %do;
				%put ERROR: OUTROC= data set not found.;
				%goto exit;
				%end;
			%end;
		%else %do;
			%put ERROR: The OUTROC= option is required.;
			%goto exit;
			%end;

	%* Verify OUT= is specified and the data set exists;
	%if %quote( &out ) ne %then %do;
			%if %sysfunc( exist( &out ) ) ne 1 %then %do;
				%put ERROR: OUT= data set not found.;
				%goto exit;
				%end;
			%end;
		%else %do;
			%put ERROR: The OUT= option is required.;
			%goto exit;
			%end;

	data _outroc;
		set &outroc;
		_prob_ = round( _prob_, &round );
		if _n_ = 1 then do;
			call symput( 'n1', put( _pos_ + _falneg_, 12. ) );
			call symput( 'n0', put( _neg_ + _falpos_, 12. ) );
			end;
	run;

	%let nobs = %eval( &n0 + &n1 );
	%let n1 = %sysfunc( putn( &n1, comma12. ) );
	  %* puts in the comma for big numbers, gets rid of blank spaces around it;
	%let n0 = %sysfunc( putn( &n0, comma12. ) );
	%let nobs = %sysfunc( putn( &nobs, comma12. ) );

	data _out;
		set &out;
		_obs_=_n_;
		if &p = . then delete;
		_prob_=round(&p , &round);
	run;

	proc sort data=_out nodupkey;
		by _prob_;
	run;

	proc sort data=_outroc nodupkey;
		by _prob_;
	run;

	data _rocplot;
		retain area 0 _sensit_o 1 _1mspec_o 1;
		_inout=0;
		_inroc=0;
		merge _outroc( in=_inroc ) _out( in=_inout );
			by _prob_;
		if not( _inout and _inroc ) then do;
			call symput( 'nomatch', 1 );
			delete;
			end;
		_sens_ = put( _sensit_, 5.3 );
		_spec_ = put( -_1mspec_+1, 5.3 );
		area + 0.5*( _sensit_o + _sensit_ )*( _1mspec_o - _1mspec_ );
		_sensit_o = _sensit_;
		_1mspec_o = _1mspec_;
   run;

	data _null_;
		set _rocplot end=eof;
		if eof then call symput( 'area', round( area, 0.0001 ) );
	run;

	%let area = &area;
	  %* trim leading white space;

	%if &nomatch=1 %then %do;
		%put ROCPLOT: Some predicted values in OUT= did not match predicted values;
		%put %str(         in OUTROC=.  Verify that you used the ROCEPS=0 option in);
		%put %str(         PROC LOGISTIC.);
		%end;

	data _add00;
		_sensit_ = 0;
		_1mspec_ = 0;
	run;

	data _rocplot;
		set _rocplot _add00;
	run;

	%let title = ROC Chart;
	%if &response ne %then %let title = ROC Chart for variable %upcase( &response );

	proc template;
		define statgraph rocPlot;
			begingraph;
				entrytitle textattrs=( size=14pt ) "&title";
				entrytitle textattrs=( weight=normal size=10pt ) "n=&nobs  (n0=&n0, n1=&n1)";
				entrytitle textattrs=( weight=normal size=10pt ) "Area under Curve = &area";
					layout overlayequated / equatetype=square
						yaxisopts=( gridDisplay=auto_on label="Sensitivity" griddisplay=on ) 
						xaxisopts=( gridDisplay=auto_on label="1 - Specificity" griddisplay=on )
						commonaxisopts=( tickvaluelist=( 0.0 0.2 0.4 0.6 0.8 1.0 ) viewmin=0 viewmax=1 );
					seriesplot y=_1mspec_ x=_1mspec_ / name="Baseline" legendlabel="Baseline" lineattrs=GraphData2;
					seriesplot y=_sensit_ x=_1mspec_ / name="Baseline2" legendlabel="Target" lineattrs=GraphData1; 
					discretelegend "Baseline2" "Baseline";
					endlayout;
				endgraph;
			end;
		run;

	ods listing gpath="&path";
		*path for the PNG files from SGRENDER below;

	ods graphics / reset=all imagename="&filename";

	proc sgrender data=_rocplot template=rocPlot;
	run;

	ods graphics off;

	footnote;

	%exit:

	options &opts;

%mend makeROC;

