dm 'output' clear;
dm 'log' clear;

%let root = C:\Nate\Research\_Papers\12.11.09 Managing Statistical Models\2013.01 SGF\Code;
%let outroot = &root\Output;

options sasautos=( "&root\Macros" ) mautosource notes nonumber nodate mcompilenote=all source source2
	linesize=180 orientation=landscape;

libname home "&root\Data";

ods listing;

data titanic_nomissing;
  * data set without missing values so that all rows can be used in our PROC LOGISTIC model below;
	set home.titanic;
	if cmiss( survived, passenger_class, sex, age, siblings_and_spouses ) = 0;
run;

ods html close;

proc surveyselect data=titanic_nomissing out=training method=srs samprate=0.60 seed=12345 noprint;
run;

proc sort data=titanic_nomissing;
	by name;
run;

proc sort data=training;
	by name;
run;

data test;
	merge titanic_nomissing ( in=a ) training ( in=b );
		by name;
	if a and not b;
run;

title 'Training Set';

proc freq data=training;
	tables survived*passenger_class*sex;
run;

title 'Test Set';

proc freq data=test;
	tables survived*passenger_class*sex;
run;

title;

* We have strong evidence of having a good partition of training and test sets;

proc logistic data=training outmodel=model;
	class passenger_class sex / param=ref;
	model survived (event='1') = passenger_class sex age siblings_and_spouses;
run;

proc logistic inmodel=model;
	score data=test out=test_scored outroc=test_roc;
run;

* Confusion matrix;

title 'Confusion Matrix';

proc freq data=test_scored; 
	table f_survived*i_survived / nocol nocum nopercent; 
	table f_survived;
run;

title;

* Cum Gain Chart;
%makeCharts( data=test_scored, response=survived, p=p_1, event=1, groups=10, plot=gain, out=gainChart, path=&outroot, filename=Gain Chart );

* Lift Chart;
%makeCharts( data=test_scored, response=survived, p=p_1, event=1, groups=10, plot=lift, path=&outroot, filename=Lift Chart );

* KS Chart;
%makeCharts( data=test_scored, response=survived, p=p_1, event=1, groups=10, plot=ks, path=&outroot, filename=KS Chart );
title;

%makeROC( outroc=test_roc, out=test_scored, p=p_1, grid=yes, response=survived, path=&outroot, filename=ROC Chart2 );
